==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/tests/gm_simpleroute.c <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation			 *
 * Copyright (c) 2001 by Myricom, Inc.					 *
 * All rights reserved.	 See the file `COPYING' for copyright notice.	 *
 *************************************************************************/

/* WARNING:
 *
 *  I tried to clean up the options parsing, but I found mysteries (e.g., see
 *  "else if (argc == 5)" below).  So I didn't do the whole job, and as a
 *  result, this program still doesn't detect invalid flags etc.
 *
 *  Somebody (David?) should fix this when possible.
 *
 *                                                       - Max 2001/06/13
 */


#if defined __STDC__ || defined WIN32
#if GM_OS_VXWORKS
#include "vxWorks.h"
#endif  /* GM_OS_VXWOKRS */
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>		/* for strtol() */

#include "gm_internal.h"

#define MAX_BOARD_ID 2000

gm_port_t *g_port = 0;
#define TP(a) fprintf (stderr, a)

static void
print_synopsis(void)
{
  TP ("\n");
  TP ("Myricom 'gm_simpleroute' sets a simple route for the specified board.\n");
}


static void
print_options (void)
{
     TP ("\nusage: gm_simpleroute [--board=board] [--loopback]\n\n");

     TP ("-B, --board=NUM   [0]   board for which the route is to be set\n");
     TP ("    --loopback    [off] route is via a loopback cable\n");
     TP ("    --help              output usage and options information, then exit\n");
#undef TP
}


static void
#ifdef va_dcl
usage (fmt, va_alist)
     const char *fmt;
     va_dcl
#else
usage (const char *fmt, ...)
#endif
{
  va_list ap;

  print_options();

/* Now print the caller's diagnostic */
#if defined __STDC__ || defined WIN32
  va_start (ap, fmt);
#else
  va_start (ap);
#endif
  vfprintf (stderr, fmt, ap);
  va_end (ap);

  if (g_port)
	  gm_close(g_port);

  gm_exit (GM_INVALID_PARAMETER);

} /* usage() */


static
void
exit_if_error (const char *msg, gm_status_t status)
{
  if (status != GM_SUCCESS)
    {
      gm_perror (msg, status);
      gm_exit (status);
    }
}

static int
gm_simpleroute (int argc, char *argv[])
{
  gm_status_t status;
  unsigned nodeid;
  int i;
  unsigned char uniqueboard[6],unique_byid[6];
  unsigned char ether[2][6];
  unsigned char route[1]={0x80};
  int routelen = 1;
  int portid = 1;
  int set_loopback = 0;
  int board_id = 0;
  int peerid = 1;
  int thisid = 1;
  int back = 0;
  char *master, *slave = 0;

  master = "localhost";


  for (argc--, argv++; argc; argc--, argv++) {
       /* Dispose of the --help flag first                                */
       if (strcmp (*argv, "--help") == 0) {
	    print_synopsis();
	    print_options();
	    gm_exit (GM_SUCCESS);
       }
       if (sscanf(*argv, "--board=%i", &board_id) ==1) {
	    if ((board_id < 0) || (board_id > MAX_BOARD_ID)) {
		 usage("bad value for option %s\n", *argv);
	    }
       }
       else if (strcmp(*argv, "-B") == 0) {
            argc--, argv++;
            if (!argc) {
                usage("Board number expected after '-B'.\n");
            }
            if (sscanf(*argv, "%i", &board_id) != 1
                    || board_id < 0
                    || board_id > MAX_BOARD_ID) {
                usage("bad board number: %d\n", board_id);
            }
            continue;
        }
        else if ((strcmp(*argv, "loopback") == 0) ||         /* Deprecated */
		 (strcmp(*argv, "loop") == 0) ||             /* Deprecated */
		 (strcmp(*argv, "l") == 0) ||                /* Deprecated */
		 (strcmp(*argv, "-loopback") == 0) ||        /* Deprecated */
		 (strcmp(*argv, "--loopback") == 0) ||
		 (strcmp(*argv, "-loop") == 0) ||            /* Deprecated */
		 (strcmp(*argv, "--loop") == 0)) {           /* Deprecated */
	    set_loopback = 1;
            continue;
        }
       else if (*argv[0] == '-') {
	    usage("Invalid option %s.\n", *argv);
       }
    }

  if (set_loopback) {
    route[0] = 0;
    routelen = 0;
    fprintf(stderr,"setting loopback route\n");
  } else if (argc == 5) {
    route[0] = 0;
    routelen = 0;
    master = argv[1];
    slave = argv[3];
    for (i=0;i<6;i++) {
      ether[0][i] = (unsigned char)strtol(strtok(i==0?argv[2]:0,":"),NULL,16);
    }
    printf("master:");
    for(i=0;i<6;i++)
      printf("%02x:",ether[0][i]);
    for (i=0;i<6;i++) {
      ether[1][i] = (unsigned char)strtol(strtok(i==0?argv[4]:0,":"),NULL,16);
    }
    printf("slave:");
    for(i=0;i<6;i++)
      printf("%02x:",ether[1][i]);
    printf("\n");
    back = 1;
    peerid = 2;
    fprintf(stderr,"setting back-to-back route\n");
  } else {
    fprintf(stderr,"setting switch round-trip route\n");
  }

  status = gm_open (&g_port, board_id, portid,
		    "small-gm-test", (enum gm_api_version) GM_API_VERSION);
  if (status != GM_SUCCESS)
    gm_perror ("open failed", status), gm_exit (status);
  status = gm_get_node_id(g_port,&nodeid);
  if (status != GM_SUCCESS) {
    if (status == GM_NODE_ID_NOT_YET_SET)
      nodeid = GM_NO_SUCH_NODE_ID;
    else
      gm_perror ("could not get node ID", status), gm_exit (status);
  }
  printf("this node id = %d, board number = %d\n",nodeid, board_id);
  status = gm_get_unique_board_id (g_port, (char*)uniqueboard);
  if (status != GM_SUCCESS)
    gm_perror ("could not get unique board id", status), gm_exit (status);
  status = gm_node_id_to_unique_id (g_port, nodeid, (char*)unique_byid);
  if (status != GM_SUCCESS) {
    gm_perror ("could not convert node ID to unique ID", status);
    gm_exit (status);
  }
  printf("uniqueid by node_id_to_unique_id =");
  for(i=0;i<6;i++)
    printf("%02x:",unique_byid[i]);
  printf("\n");
  printf("uniqueid=");
  for(i=0;i<6;i++)
    printf("%02x:",uniqueboard[i]);
  printf("\n");
  if (back == 1) {
    if (memcmp(ether[0],uniqueboard,6) == 0)
      thisid = 1;
    else if (memcmp(ether[1],uniqueboard,6) == 0)
      thisid = 2;
    else
      gm_perror ("obscure error", GM_FAILURE), gm_exit (GM_FAILURE);
  }
  status = _gm_set_node_id (g_port, thisid);
  status = _gm_set_unique_id (g_port, 1, (char *)uniqueboard);
  exit_if_error ("could not set unique ID 1", status);

  exit_if_error ("could not set node ID", status);
  if (back) {
    status = _gm_set_unique_id (g_port, 1, (char *)ether[0]);
    exit_if_error ("could not set unique ID 1", status);
    status = _gm_set_unique_id (g_port, 2, (char *)ether[1]);
    exit_if_error ("could not set unique ID 2", status);
  }
  status = _gm_set_host_name (g_port, 1, master);
  exit_if_error ("could not set host name 1", status);
  if (back) {
    status = _gm_set_host_name (g_port, 2, slave);
    exit_if_error ("could not set host name 2", status);
  }
  status = _gm_set_route (g_port, peerid, routelen, (char *)route);
  exit_if_error ("could not set route to peer", status);

  if (g_port) {
    gm_close (g_port);
  }
  return(status);
}

/****************************************************************
 * entry point
 ****************************************************************/

#define GM_MAIN gm_simpleroute
#include "gm_main.h"

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/tests/gm_simpleroute.c <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation			 *
 * Copyright (c) 2001 by Myricom, Inc.					 *
 * All rights reserved.	 See the file `COPYING' for copyright notice.	 *
 *************************************************************************/

/* WARNING:
 *
 *  I tried to clean up the options parsing, but I found mysteries (e.g., see
 *  "else if (argc == 5)" below).  So I didn't do the whole job, and as a
 *  result, this program still doesn't detect invalid flags etc.
 *
 *  Somebody (David?) should fix this when possible.
 *
 *                                                       - Max 2001/06/13
 */


#if defined __STDC__ || defined WIN32
#if GM_OS_VXWORKS
#include "vxWorks.h"
#endif  /* GM_OS_VXWOKRS */
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>		/* for strtol() */

#include "gm_internal.h"

#define MAX_BOARD_ID 2000

gm_port_t *g_port = 0;
#define TP(a) fprintf (stderr, a)

static void
print_synopsis(void)
{
  TP ("\n");
  TP ("Myricom 'gm_simpleroute' sets a simple route for the specified board.\n");
}


static void
print_options (void)
{
     TP ("\nusage: gm_simpleroute [--board=board] [--loopback]\n\n");

     TP ("-B, --board=NUM   [0]   board for which the route is to be set\n");
     TP ("    --loopback    [off] route is via a loopback cable\n");
     TP ("    --help              output usage and options information, then exit\n");
#undef TP
}


static void
#ifdef va_dcl
usage (fmt, va_alist)
     const char *fmt;
     va_dcl
#else
usage (const char *fmt, ...)
#endif
{
  va_list ap;

  print_options();

/* Now print the caller's diagnostic */
#if defined __STDC__ || defined WIN32
  va_start (ap, fmt);
#else
  va_start (ap);
#endif
  vfprintf (stderr, fmt, ap);
  va_end (ap);

  if (g_port)
	  gm_close(g_port);

  gm_exit (GM_INVALID_PARAMETER);

} /* usage() */


static
void
exit_if_error (const char *msg, gm_status_t status)
{
  if (status != GM_SUCCESS)
    {
      gm_perror (msg, status);
      gm_exit (status);
    }
}

static int
gm_simpleroute (int argc, char *argv[])
{
  gm_status_t status;
  unsigned nodeid;
  int i;
  unsigned char uniqueboard[6],unique_byid[6];
  unsigned char ether[2][6];
  unsigned char route[1]={0x80};
  int orig_argc;
  int routelen = 1;
  int portid = 1;
  int set_loopback = 0;
  int board_id = 0;
  int peerid = 1;
  int thisid = 1;
  int back = 0;
  char *master, *slave = 0;
  char **orig_argv;
  
  master = "localhost";

  orig_argc = argc;
  orig_argv = argv;
  
  for (argc--, argv++; argc; argc--, argv++) {
       /* Dispose of the --help flag first                                */
       if (strcmp (*argv, "--help") == 0) {
	    print_synopsis();
	    print_options();
	    gm_exit (GM_SUCCESS);
       }
       if (sscanf(*argv, "--board=%i", &board_id) ==1) {
	    if ((board_id < 0) || (board_id > MAX_BOARD_ID)) {
		 usage("bad value for option %s\n", *argv);
	    }
       }
       else if (strcmp(*argv, "-B") == 0) {
            argc--, argv++;
            if (!argc) {
                usage("Board number expected after '-B'.\n");
            }
            if (sscanf(*argv, "%i", &board_id) != 1
                    || board_id < 0
                    || board_id > MAX_BOARD_ID) {
                usage("bad board number: %d\n", board_id);
            }
            continue;
        }
        else if ((strcmp(*argv, "loopback") == 0) ||         /* Deprecated */
		 (strcmp(*argv, "loop") == 0) ||             /* Deprecated */
		 (strcmp(*argv, "l") == 0) ||                /* Deprecated */
		 (strcmp(*argv, "-loopback") == 0) ||        /* Deprecated */
		 (strcmp(*argv, "--loopback") == 0) ||
		 (strcmp(*argv, "-loop") == 0) ||            /* Deprecated */
		 (strcmp(*argv, "--loop") == 0)) {           /* Deprecated */
	    set_loopback = 1;
            continue;
        }
       else if (*argv[0] == '-') {
	    usage("Invalid option %s.\n", *argv);
       }
    }
  argc = orig_argc;
  argv = orig_argv;
  
  if (set_loopback) {
    route[0] = 0;
    routelen = 0;
    fprintf(stderr,"setting loopback route\n");
  } else if (argc == 5) {
    route[0] = 0;
    routelen = 0;
    master = argv[1];
    slave = argv[3];
    for (i=0;i<6;i++) {
      ether[0][i] = (unsigned char)strtol(strtok(i==0?argv[2]:0,":"),NULL,16);
    }
    printf("master:");
    for(i=0;i<6;i++)
      printf("%02x:",ether[0][i]);
    for (i=0;i<6;i++) {
      ether[1][i] = (unsigned char)strtol(strtok(i==0?argv[4]:0,":"),NULL,16);
    }
    printf("slave:");
    for(i=0;i<6;i++)
      printf("%02x:",ether[1][i]);
    printf("\n");
    back = 1;
    peerid = 2;
    fprintf(stderr,"setting back-to-back route\n");
  } else {
    fprintf(stderr,"setting switch round-trip route\n");
  }

  status = gm_open (&g_port, board_id, portid,
		    "small-gm-test", (enum gm_api_version) GM_API_VERSION);
  if (status != GM_SUCCESS)
    gm_perror ("open failed", status), gm_exit (status);
  status = gm_get_node_id(g_port,&nodeid);
  if (status != GM_SUCCESS) {
    if (status == GM_NODE_ID_NOT_YET_SET)
      nodeid = GM_NO_SUCH_NODE_ID;
    else
      gm_perror ("could not get node ID", status), gm_exit (status);
  }
  printf("this node id = %d, board number = %d\n",nodeid, board_id);
  status = gm_get_unique_board_id (g_port, (char*)uniqueboard);
  if (status != GM_SUCCESS)
    gm_perror ("could not get unique board id", status), gm_exit (status);
  status = gm_node_id_to_unique_id (g_port, nodeid, (char*)unique_byid);
  if (status != GM_SUCCESS) {
    gm_perror ("could not convert node ID to unique ID", status);
    gm_exit (status);
  }
  printf("uniqueid by node_id_to_unique_id =");
  for(i=0;i<6;i++)
    printf("%02x:",unique_byid[i]);
  printf("\n");
  printf("uniqueid=");
  for(i=0;i<6;i++)
    printf("%02x:",uniqueboard[i]);
  printf("\n");
  if (back == 1) {
    if (memcmp(ether[0],uniqueboard,6) == 0)
      thisid = 1;
    else if (memcmp(ether[1],uniqueboard,6) == 0)
      thisid = 2;
    else
      gm_perror ("obscure error", GM_FAILURE), gm_exit (GM_FAILURE);
  }
  status = _gm_set_node_id (g_port, thisid);
  status = _gm_set_unique_id (g_port, 1, (char *)uniqueboard);
  exit_if_error ("could not set unique ID 1", status);

  exit_if_error ("could not set node ID", status);
  if (back) {
    status = _gm_set_unique_id (g_port, 1, (char *)ether[0]);
    exit_if_error ("could not set unique ID 1", status);
    status = _gm_set_unique_id (g_port, 2, (char *)ether[1]);
    exit_if_error ("could not set unique ID 2", status);
  }
  
  {
    gm_u8_t unique[6];
    
    status = gm_node_id_to_unique_id (g_port, nodeid, unique);
    exit_if_error ("Could not get unique ID for this node", status);
    
    status = _gm_set_route (g_port, unique, routelen, (char *)route);
    exit_if_error ("could not set route to peer", status);
  }
  
  if (g_port) {
    gm_close (g_port);
  }
  return(status);
}

/****************************************************************
 * entry point
 ****************************************************************/

#define GM_MAIN gm_simpleroute
#include "gm_main.h"
